package Blatt02_DiningSavages;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Ein Topf mit veränderlichem Füllstand. Ein neu erzeugter Topf ist zunächst leer.
 * 
 * @author Sebastian Flothow (sebastian@flothow.de)
 */
public class Pot {
	private AtomicInteger content = new AtomicInteger(0);
	
	protected final Lock lock = new ReentrantLock();
	protected final Condition potNotEmpty = lock.newCondition();
	protected final Condition potIsEmpty = lock.newCondition();
	
	/**
	 * Gibt zurück, wieviele Portionen der Topf momentan enthält.
	 * 
	 * @return Anzahl der Portionen
	 */
	public int getContent() { return content.get(); }
	
	/**
	 * Nimmt eine Portion aus dem Topf heraus. Dies ist nur möglich, 
	 * wenn der Topf mindestens eine Portion enthält.
	 * 
	 * @throws IllegalStateException wenn der Topf leer ist
	 */
	public void takeServing() {
		if (content.get() <= 0) {
			throw new IllegalStateException(
					"Aus einem leeren Topf kann man nichts rausnehmen");
		}
		
		content.decrementAndGet();
	}
	
	/**
	 * Füllt Portionen in den Topf. Dies ist nur möglich, wenn der Topf leer 
	 * ist.
	 * 
	 * @param n Anzahl der einzufüllenden Portionen
	 * 
	 * @throws IllegalStateException wenn der Topf nicht leer ist
	 */
	public void putServings(int n) {
		if (content.get() != 0) {
			throw new IllegalStateException(
					"Der Topf darf nur nachgefüllt werden wenn er leer ist");
		}
		
		content.addAndGet(n);
	}
}
