package Blatt02_DiningSavages;




/**
 * Implementiert einen Wilden als eigenständigen Thread.
 * 
 * @author Sebastian Flothow (sebastian@flothow.de)
 */
public class Savage extends Thread {
	private int i;
	private Pot pot;
	
	/**
	 * Erzeugt einen Wilden, der aus dem angegebenen Topf isst.
	 * 
	 * @param i eindeutige Nummer des Wilden, wird in Ausgaben verwendet
	 * @param pot Topf, aus dem der Wilde isst
	 */
	public Savage(int i, Pot pot) {
		this.i = i;
		this.pot = pot;
	}
	
	/**
	 * Isst aus dem Topf. Nimmt in einer Endlosschleife jeweils eine Portion aus
	 * dem Topf und isst sie. Falls eine Exception auftritt wird diese auf 
	 * stderr ausgegeben und der Thread beendet.
	 */
	@Override
	public void run() {
		try {
			while (true) {
				/* Savages need synchronized access to the pot.
				 * If the pod is empty, we need a lock to cry to the cook 
				 * "where the hell is the food" ( .signal() ).
				 * after that we wait until the cook cry "there it is"
				 * and the savage can get his food,
				 * before the next savage can get it
				 * you have to unlock the lock ;-)
				 */
				synchronized ( pot ){
					if ( this.pot.getContent() == 0 ){
						this.pot.lock.lock();
						this.pot.potIsEmpty.signal();
						this.pot.potNotEmpty.await();
						this.pot.lock.unlock();
					}	
					pot.takeServing();
				}

				System.out.println(
					"Wilder " + i + " nimmt eine Portion aus dem Topf");
				eat();
			}
		} catch (Exception e) {
			System.err.print("Wilder " + i + ": ");
			e.printStackTrace();
		} finally { this.pot.lock.unlock(); }
	}	
	
	protected void eat() throws InterruptedException {
		System.out.println("Wilder " + i + " isst");
		Thread.sleep(5000);
	}
}
